@@ -9,15 +9,16 @@ from typing import Optional, List, Dict, Tuple, Union, Type
 
 import networkx as nx
 import numpy as np
+import overpy
 import pandas as pd
-from ipyleaflet import Map, ScaleControl, FullScreenControl, Polyline, Icon, Marker, Circle, TileLayer, LayerGroup
+from ipyleaflet import Map, ScaleControl, FullScreenControl, Polyline, Marker, Circle, LayerGroup
 from ipywidgets import HTML
 from pandas.io.sql import DatabaseError as PandasDatabaseError
 from scipy.spatial import KDTree
 from scipy.stats import norm
 
 from pyridy import config
-from pyridy.osm import OSM
+from pyridy.osm import OSM, OSMRailwayLine
 from pyridy.osm.utils import is_point_within_line_projection, project_point_onto_line
 from pyridy.utils import Sensor, AccelerationSeries, LinearAccelerationSeries, MagnetometerSeries, OrientationSeries, \
     GyroSeries, RotationSeries, GPSSeries, PressureSeries, HumiditySeries, TemperatureSeries, WzSeries, LightSeries, \
@@ -140,8 +141,9 @@ class RDYFile:
 
         # OSM Data (set by Campaign)
         self.osm: Optional[OSM] = None
-        self.matched_nodes = []  # Nodes from Map Matching
-        self.matched_ways = []  # Ways from Map Matching
+        self.matched_nodes: Optional[List[overpy.Node]] = []  # Nodes from Map Matching
+        self.matched_ways: Optional[List[overpy.Way]] = []  # Ways from Map Matching
+        self.matched_line: Optional[OSMRailwayLine] = None  # Matched Railway Line
 
         if self.path:
             self.load_file(self.path)
@@ -513,6 +515,23 @@ class RDYFile:
                 self.matched_ways = list(set([self.osm.way_dict[w_id] for w_id in m_w_ids]))  # Mapped Ways
                 logger.debug("(%s) Found %d nodes that match the files GPS track!" % (self.filename,
                                                                                       len(self.matched_nodes)))
+
+                if len(self.matched_nodes) > 0:
+                    m_ratios = [len(set(self.matched_nodes).intersection(line.nodes)) / len(self.matched_nodes) for line
+                                in
+                                self.osm.railway_lines if self]
+
+                    if max(m_ratios) > config.options["MAP_MATCHING_MIN_LINE_MATCH_RATIO"]:
+                        self.matched_line = self.osm.railway_lines[m_ratios.index(max(m_ratios))]
+                        logger.debug("(%s) Matched Railway Line: %s (%.f Match Ratio)" % (self.filename,
+                                                                                          self.matched_line.name,
+                                                                                          max(m_ratios)))
+                    else:
+                        logger.debug("(%s) Could not match map matching results to a specific railway line" %
+                                     self.filename)
+                else:
+                    logger.debug("(%s) Could not match map matching results to a specific railway line" %
+                                 self.filename)
         else:
             logger.warning("(%s) Can't do map matching since no file contains no OSM data" % self.filename)
         pass
