@@ -193,7 +193,7 @@ class OSMRelation:
                 ways = list(set(list(itertools.chain.from_iterable([n.ways for n in nodes]))))
                 self.tracks.append(OSMTrack(nodes, ways))
             except nx.NetworkXNoPath as e:
-                print(e)
+                logger.debug(e)
 
         logger.debug("Number of individual tracks: %d" % len(self.tracks))
 
