@@ -1,3 +1,4 @@
+import itertools
 import logging
 from datetime import timedelta
 from typing import Union
@@ -5,9 +6,13 @@ from typing import Union
 import numpy as np
 import pandas as pd
 from scipy import signal, integrate
+from shapely.geometry import LineString, Point
+from shapely.ops import nearest_points
 from tqdm.auto import tqdm
 
-from pyridy import Campaign
+from pyridy import Campaign, config
+from pyridy.file import RDYFile
+from pyridy.osm.utils import convert_way_to_line_string, OSMResultNode
 from pyridy.processing import PostProcessor
 from pyridy.utils import LinearAccelerationSeries, GPSSeries
 
@@ -15,7 +20,8 @@ logger = logging.getLogger(__name__)
 
 
 class ExcitationProcessor(PostProcessor):
-    def __init__(self, campaign: Campaign, f_s: int = 200, f_c: float = 0.1, order: int = 4):
+    def __init__(self, campaign: Campaign, f_s: int = 200, f_c: float = 0.1, order: int = 4,
+                 p_thres: float = .005, p_dist: int = 100, osm_integration=True):
         """ The ExcitationProcessor performs a double integration of the acceleration data to calculate
         excitations. High-Pass Filters are applied to remove static offset and drift. Hence, the resulting
         excitations only represent high-frequent excitations but no quasi-static movements
@@ -37,6 +43,10 @@ class ExcitationProcessor(PostProcessor):
         self.f_c = f_c
         self.order = order
 
+        self.p_thres = p_thres
+        self.p_dist = p_dist
+        self.osm_integration = osm_integration
+
         self.b, self.a = signal.butter(self.order, 2 * self.f_c / self.f_s, 'high')  # High Pass (2*f_c/f_s)
 
     def execute(self, axes: Union[str, list] = "z", intp_gps: bool = True):
@@ -61,10 +71,11 @@ class ExcitationProcessor(PostProcessor):
         else:
             raise ValueError("axes must be list or str")
 
+        f: RDYFile
         for f in tqdm(self.campaign):
             if len(f.measurements[LinearAccelerationSeries]) == 0:
-                logger.warning("(%s) LinearAccelerationSeries is empty, can't execute ExcitationProcessor on this file"
-                               % f.filename)
+                logger.warning("({f.filename}) LinearAccelerationSeries is empty, can't execute ExcitationProcessor "
+                               "on this file")
                 continue
             else:
                 lin_acc_df = f.measurements[LinearAccelerationSeries].to_df()
@@ -102,6 +113,39 @@ class ExcitationProcessor(PostProcessor):
                     lin_s_hp = signal.filtfilt(self.b, self.a, lin_s, padlen=150)
                     df["lin_s_" + ax] = lin_s_hp
 
+                    # Peak nodes for OSM
+                    if self.osm_integration and len(f.measurements[GPSSeries]) > 0:
+                        if self.campaign.osm:
+                            lin_s_abs = np.abs(lin_s_hp)
+
+                            p_idxs, prop = signal.find_peaks(lin_s_hp, height=self.p_thres, distance=self.p_dist)
+                            peaks = lin_s_abs[p_idxs]
+                            lons, lats = df.lon[p_idxs], df.lat[p_idxs]
+                            xs, ys = config.proj(lons, lats)
+                            if f.matched_line:
+                                trk = f.matched_line.tracks[0]
+                                way_lines = [convert_way_to_line_string(w, frmt="x,y") for w in trk.ways]
+                                for i, coord in enumerate(zip(xs, ys)):
+                                    p = Point(*coord)
+
+                                    # Get the closest way to point
+                                    dists = [line.distance(p) for line in way_lines]
+                                    min_d = min(dists)
+                                    if min_d <= config.options["RESULT_MATCHING_MAX_DISTANCE"]:
+                                        line = way_lines[dists.index(min_d)]
+                                        way = trk.ways[dists.index(min_d)]
+                                        pp = line.interpolate(line.project(p))  # Projection of GPS point to OSM line
+                                        lon, lat = config.proj(pp.x, pp.y, inverse=True)
+                                        r_node = OSMResultNode(lon, lat, peaks[i], f, proc=self, dir=ax)
+                                        if "results" not in way.attributes:
+                                            way.attributes["results"] = [r_node]
+                                        else:
+                                            way.attributes["results"].append(r_node)
+
+                        else:
+                            logger.warning("(%s) Campaign contains no OSM data, can't integrate results" % f.filename)
+                        pass
+
                 if ExcitationProcessor not in self.campaign.results:
                     self.campaign.results[ExcitationProcessor] = {f.filename: df}
                 else:
